#include "types.h"
#include "SdCard.h"
#include "Flash.h"

#define HEX_FILE_NAME "BootImg.asr"
#define FILE_READ 0x1
#define BUFFER_SIZE 60


extern sfr sbit TEST_LED;
extern sfr sbit TEST_LED_PORT;
extern sfr sbit USER_BUTTON_DETECT;
extern sfr sbit CARD_DETECT;


//----------------------- functions prototypes ---------------------------------
enResult pharseHexLine(hexLine_t *ptrHexLineObj,char *ptrRawHexLine );
//void soundBip();
//------------------------------------------------------------------------------

enResult getHexAsciiLine( char *ptrHexLineBuffer){
    static long unsigned int cursorPositionInFile_ulint = 0;
    short err_short = 0;
    short filehandle_intern;
    unsigned char receivedByte = 0;
    int cnt_int = 0;


    filehandle_intern =  FAT32_Open(HEX_FILE_NAME ,FILE_READ);
    err_short = FAT32_Read(filehandle_intern, &receivedByte, 1);                // read one byte into filebuffer, we expect a ":"
    *ptrHexLineBuffer = receivedByte;
    ptrHexLineBuffer++;
    cnt_int++;
    cursorPositionInFile_ulint++;
    err_short = FAT32_Seek(filehandle_intern, cursorPositionInFile_ulint);      // increase cursor one step in File
    do{
      err_short = FAT32_Read(filehandle_intern, &receivedByte, 1);              // read one byte into filebuffer
      if( receivedByte == 0x3A )                                                // if received char is ":" break and dont save it
          break;
      *ptrHexLineBuffer = receivedByte;
      ptrHexLineBuffer++;
      cursorPositionInFile_ulint++;
      cnt_int++;
      err_short = FAT32_Seek(filehandle_intern, cursorPositionInFile_ulint);
    }while( cnt_int < 45 );                                                     // stay in loop until max buffer size
   
    FAT32_Close(filehandle_intern);
    return  eResultOk;
}





unsigned short  isBootloaderMode() {
  unsigned short res;
  if(USER_BUTTON_DETECT == 1){  // if button is pressed
     Delay_ms(500);             // wait if button is still pressed
         
    if( (USER_BUTTON_DETECT == BUTTON_PRESSED) && (CARD_DETECT == SD_CARD_MOUNTED) ){   // if button is still pressed & Sd card inserted

              res = FAT32_Exists(HEX_FILE_NAME);
              return (res == 1)? TRUE : FALSE ;
           } else {
               return FALSE;
       }

   }
     
}
//------------------------------------------------------------------------------
unsigned  short getByteBinValue(char * ptrFirstHexChar){
   unsigned short1 = 0;
   unsigned short2 = 0;
   static unsigned short lut[] = { 0,0,0,0,0,0,0,0,0,0,
                                   0,0,0,0,0,0,0,0,0,0,
                                   0,0,0,0,0,0,0,0,0,0,
                                   0,0,0,0,0,0,0,0,0,0,
                                   0,0,0,0,0,0,0,0,0,1,
                                   2,3,4,5,6,7,8,9,0,0,
                                   0,0,0,0,0,10,11,12,13,14,
                                   15
                                 };
       short1 = lut[ptrFirstHexChar[0]] << 4;
       short2 = lut[ptrFirstHexChar[1]];
       return short1 | short2;
       //return ( (lut[ptrFirstHexChar[0]] << 4) | lut[ptrFirstHexChar[1]] );
}
//------------------------------------------------------------------------------
enResult pharseHexLine(hexLine_t *ptrHexLineObj,char *ptrRawHexLine ){
unsigned short checkSum = 0;
unsigned short i;
   if( ptrRawHexLine[0] != ':'){
      return eErrNoDelimiter;
   }

    ptrHexLineObj->dataLength = getByteBinValue(&ptrRawHexLine[1]);
    ptrHexLineObj->addrOffset = ((unsigned long)getByteBinValue(&ptrRawHexLine[3])) <<8 ;
    ptrHexLineObj->addrOffset += ((unsigned long)getByteBinValue(&ptrRawHexLine[5]));
    ptrHexLineObj->type       =  getByteBinValue(&ptrRawHexLine[7]);
    ptrHexLineObj->checkSum   =  getByteBinValue(&ptrRawHexLine[9+2*ptrHexLineObj->dataLength]);
    if( ptrHexLineObj->type == HEX_TYPE_DATA){
       for (i = 0; i < ptrHexLineObj->dataLength; i++){
          ptrHexLineObj->hexBinData[i] = getByteBinValue(&ptrRawHexLine[9+i+i]); // addiing is faster then 2*i
       }
    } else if (ptrHexLineObj->type == HEX_TYPE_LINEAR_SEGMENT){
       ptrHexLineObj->addrSegment = 0L; // just to be on safe side
       //update the segment address with the new value from date field
       ptrHexLineObj->addrSegment = (unsigned long)getByteBinValue( &ptrRawHexLine[9]) << 8;
       ptrHexLineObj->addrSegment += (unsigned long)getByteBinValue( &ptrRawHexLine[11]);
       ptrHexLineObj->addrSegment = ptrHexLineObj->addrSegment << 16;
    } else{
       //do nothing
    }
    // verify that after the data we have LF and CR (0xD 0xD)
    if( (ptrRawHexLine[9+2*ptrHexLineObj->dataLength+2] != 0xD) || (ptrRawHexLine[9+2*ptrHexLineObj->dataLength+3 ] != 0xA)){
       return eParseFailed;
    } else {
       return eResultOk;
    }
    // check the checksum  (TO DO)
    
}
//------------------------------------------------------------------------------
enResult runBootloader(){
  enResult eResult;
  static unsigned long currentAddrSegment = 0;
  char hexRawLine[1+2+4+2+32+2+2];//1-start of  row, 2-num data bytes, 4-address, 2-type, 32- ascii data, 2- checksum, 2 <LF><CR>
  hexLine_t parsedHexLine;
  short filehandle;
  int i = 0;
 
  filehandle =  FAT32_Open(HEX_FILE_NAME ,FILE_READ);
  if (filehandle < 0){
     return eErrResult;
  }
  // if we are here hex file found and opened syccessfuly
  do{
     eResult = getHexAsciiLine(hexRawLine);
     if(eResult == eResultOk){
        eResult = pharseHexLine(& parsedHexLine,hexRawLine );
        if(eResult != eResultOk){
            return eParseFailed;
        }

        if (i++ % 300 == 0){
           Sound_Play(4000,20); // some indication for the user
        }
        
        switch (parsedHexLine.type){
           case HEX_TYPE_DATA:
              parsedHexLine.realAddess = currentAddrSegment + parsedHexLine.addrOffset;
              if ( (parsedHexLine.realAddess == 0x1FC00380) || (parsedHexLine.realAddess == 0x1FC02FF0) ){ // skip BootVT programming
              /*
                 we enable writing of the SysVT which is the IVT but not BootVT since this will prevent the BL application to
                 start.
                 if we want to enable writing the configuration bits we should allow the writing of the hex file 
                 with the address of 0x1C02FF0 ( phy of 0xBF0C2FF0 )
              */

              }else{
                 writeDataToShadowOfFlashBlock(& parsedHexLine);
              }
              break;
           case HEX_TYPE_LINEAR_SEGMENT:
              currentAddrSegment = parsedHexLine.addrSegment;
              break;
           case HEX_TYPE_END_FILE:
              writeShadowToFlash(); // flush the last shadow block to flash memory
              FAT32_Close(filehandle);
              return eResultOk; // finished loading user program
              break; 
           default:
              break;
        }
     } else {
        return eErrReadHexLine;
     }
  } while ((eResult == eResultOk) && (parsedHexLine.type !=  HEX_TYPE_END_FILE));
}
//------------------------------------------------------------------------------
